package org.openjdk.shenandoah.wip;

import org.openjdk.jmh.annotations.*;

import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.concurrent.ThreadLocalRandom;
import java.util.concurrent.TimeUnit;

@Warmup(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Measurement(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Fork(1)
@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.NANOSECONDS)
@Threads(1)
@State(Scope.Benchmark)
public class WeakRefs {

    @Param({"1", "10", "100", "1000", "10000"})
    int count;

    ReferenceQueue<Target> refq;
    Target[] targets;
    MyRef[] refs;

    @Setup
    public void setup() {
        refq = new ReferenceQueue<>();
        targets = new Target[count];
        refs = new MyRef[count];
        for (int c = 0; c < count; c++) {
            Target o = new Target();
            targets[c] = o;
            refs[c] = new MyRef(refq, o, c);
        }
    }

    @Benchmark
    public void test() {
        targets[ThreadLocalRandom.current().nextInt(count)] = null;

        MyRef r;
        while ((r = (MyRef)refq.poll()) != null) {
            Target o = new Target();
            targets[r.index] = o;
            refs[r.index] = new MyRef(refq, o, r.index);
        }
    }

    static class MyRef extends WeakReference<Target> {
        int index;
        public MyRef(ReferenceQueue<Target> refq, Target obj, int idx) {
            super(obj, refq);
            this.index = idx;
        }
    }

    static class Target {

    }

}
